function book_setup(ifmissing)
% book_setup()
% book_setup('warn')
% book_setup('error')
% book_setup('ignore')
%
% Adds the necessary directories to Octave/Matlab's path and checks for
% Casadi and MPCTools. Argument specifies what to do if Casadi/MPCTools are not
% found.
narginchk(0, 1);
if nargin() < 1
    ifmissing = 'warn';
end

% Add the booklib path.
thisdir = fileparts(mfilename('fullpath'));
addpath([thisdir, '/booklib']);

% Bug in dasrt octave version 4.2.1
if isOctave() && isequal(version(), '4.2.1')
    dasrt_path = [thisdir, '/booklib/dasrt_v4_2_1.oct'];
    if exist(dasrt_path, 'file')
        autoload('dasrt', dasrt_path);
    else
        warning(['dasrt is bugged in Octave 4.2. ', ...
                 'Scripts calling dasrt may fail.']);
    end
end

% Look for MPCTools, Casadi, and Sundials.
directories = {'casadi', 'mpctools'};
for i = 1:length(directories)
    directory = [thisdir, '/', directories{i}];
    if exist(directory, 'dir')
        addpath(directory);
    end
end

% Issue errors, warnings, etc.
switch ifmissing
case 'error'
    missing = @error;
case 'warn'
    missing = @warning;
case 'ignore'
    missing = @(varargin) [];
otherwise
    error('Invalid argument!');
end

if isempty(which('casadiMEX'))
    missing(['Casadi not found. Download from files.casadi.org and unzip ' ...
             'to a folder called ''casadi''.']);
end

if isempty(which('import_mpctools'))
    missing(['MPCTools not found. Download from bitbucket.org/' ...
             'rawlings-group/octave-mpctools and unzip to a folder called ' ...
             '''mpctools''.']);
end

end%function

